var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};
var _createContainerParams, _manifest, _env, _root, _modToRoute;
import { posix } from "node:path";
import { fileURLToPath } from "node:url";
import { createContainer } from "../dev/index.js";
import { openConfig, createSettings } from "../config/index.js";
import { createViteLoader } from "../module-loader/index.js";
import { createRouteManifest } from "../routing/index.js";
import {
  createDevelopmentEnvironment,
  getScriptsAndStyles,
  preload
} from "../render/dev/index.js";
import {
  renderPage as coreRenderPage
} from "../render/index.js";
import { App } from "./index.js";
import { nodeLogDestination } from "../logger/node.js";
const logging = {
  dest: nodeLogDestination,
  level: "error"
};
class DevApp extends App {
  constructor(params) {
    const { root, userConfig } = params;
    const manifest = {
      adapterName: "development",
      base: userConfig == null ? void 0 : userConfig.base,
      routes: [],
      markdown: {
        contentDir: root
      },
      pageMap: /* @__PURE__ */ new Map(),
      renderers: [],
      entryModules: new Proxy({}, {
        get(target, key) {
          return key;
        }
      }),
      assets: /* @__PURE__ */ new Set(),
      propagation: /* @__PURE__ */ new Map(),
      trailingSlash: (userConfig == null ? void 0 : userConfig.trailingSlash) ?? "ignore"
    };
    super(manifest, true);
    __privateAdd(this, _createContainerParams, void 0);
    __privateAdd(this, _manifest, void 0);
    __privateAdd(this, _env, null);
    __privateAdd(this, _root, void 0);
    __privateAdd(this, _modToRoute, /* @__PURE__ */ new Map());
    this.container = null;
    this.renderPage = async (mod, ctx, env) => {
      const route = __privateGet(this, _modToRoute).get(mod);
      const { scripts, links, styles, propagationMap } = await getScriptsAndStyles({
        env: __privateGet(this, _env),
        filePath: new URL(route.component, __privateGet(this, _root))
      });
      ctx.scripts = scripts;
      ctx.links = links;
      ctx.styles = styles;
      ctx.propagation = propagationMap;
      return coreRenderPage(mod, ctx, env);
    };
    __privateSet(this, _manifest, manifest);
    __privateSet(this, _root, root);
    __privateSet(this, _createContainerParams, params);
  }
  get loaded() {
    return !!this.container;
  }
  url(pathname) {
    if (!this.loaded) {
      return void 0;
    }
    const { host, port } = this.container.settings.config.server;
    return new URL(pathname, `http://${host}:${port}`).toString();
  }
  async load() {
    if (this.loaded) {
      await this.close();
      this.container = null;
      __privateSet(this, _env, null);
    }
    const params = {
      ...__privateGet(this, _createContainerParams)
    };
    if (!__privateGet(this, _createContainerParams).userConfig) {
      const configResult = await openConfig({
        cmd: "dev",
        logging
      });
      params.settings = createSettings(configResult.astroConfig);
    }
    const container = this.container = await createContainer(params);
    __privateGet(this, _manifest).trailingSlash = container.settings.config.trailingSlash;
    const loader = createViteLoader(container.viteServer);
    const routeManifest = createRouteManifest({
      settings: container.settings,
      fsMod: __privateGet(this, _createContainerParams).fs
    }, container.logging);
    const routes = routeManifest.routes.map((routeData) => {
      return {
        routeData,
        file: routeData.component,
        links: [],
        scripts: []
      };
    });
    this.updateRoutes(routes);
    __privateSet(this, _env, createDevelopmentEnvironment(container.settings, container.logging, loader));
    return this;
  }
  async close() {
    var _a;
    await ((_a = this.container) == null ? void 0 : _a.close());
  }
  fileChanged(path) {
    const container = this.container;
    const fs = __privateGet(this, _createContainerParams).fs;
    const root = fileURLToPath(__privateGet(this, _root));
    const fullPath = posix.join(root, path);
    container.viteServer.watcher.emit("change", fullPath);
    if (!fileURLToPath(container.settings.config.root).startsWith("/")) {
      const drive = fileURLToPath(container.settings.config.root).slice(0, 2);
      container.viteServer.watcher.emit("change", drive + fullPath);
    }
  }
  handle(req, res) {
    this.container.handle(req, res);
  }
  async render(request, route) {
    if (!this.loaded) {
      await this.load();
    }
    if (!route) {
      route = this.match(request, { matchNotFound: false });
    }
    if (route) {
      const filePath = new URL(route.component, __privateGet(this, _root));
      const [renderers, mod] = await preload({
        env: __privateGet(this, _env),
        filePath
      });
      __privateGet(this, _manifest).renderers.length = 0;
      __privateGet(this, _manifest).renderers.push(...renderers);
      __privateGet(this, _manifest).pageMap.set(route.component, mod);
      __privateGet(this, _modToRoute).set(mod, route);
    }
    return super.render(request, route);
  }
}
_createContainerParams = new WeakMap();
_manifest = new WeakMap();
_env = new WeakMap();
_root = new WeakMap();
_modToRoute = new WeakMap();
export {
  DevApp,
  logging
};
